#! /bin/sh 
#-----------------------------------------------------------------
#Shell script to add local information to configuration file
#-----------------------------------------------------------------
BASE=$1
. `dirname $0`/../shell/version.sh
#-----------------------------------------------------
#loading the variable value
. $BASE/etc/$APPNAME-$VERSION/$APPNAME.conf
#-----------------------------------------------------
#Updating clement httpd configuration file
export hostipv4=`dig \`uname -n\` A +noall +short`;
export hostipv6=`dig \`uname -n\` AAAA +noall +short`;
if [ -z "$domain" ] ; then
  export domain=`dnsdomainname`
  fi

#to make sure we have at least an hostipv4 reference
if [ -z "$hostipv4" -a -z "$hostipv6" ] ; then
  export hostipv4=`grep \`uname -n\` /etc/hosts | cut  -f1`
  fi
#-----------------------------------------------------------------
add_web()

{
#converting '.' to '\.'
reldom=`echo $domain | sed -e "s/\\./\\\\\\./g"`

cat >> $BASE/etc/httpd/conf.d/$APPNAME.conf <<!EOT
#$APPNAME START
#Done by the $APPNAME first start

<VirtualHost *:80>
	ServerAlias		clement.$domain
        ServerName		clement-prod.$domain
        ServerAdmin		root@$domain
	RewriteEngine		on
	#comment it in if you want to debug redirect
	#LogLevel                alert rewrite:trace3
        RewriteCond		%{HTTP_HOST} ^clement\.$reldom				[OR]
	RewriteCond		%{HTTP_HOST} ^clement-prod\.$reldom
        RewriteRule		^(.*)$ https://clement-prod.$domain	[R=301]
        ErrorLog		logs/clemerr.log
        TransferLog		logs/clemlog.log
</VirtualHost>

<VirtualHost *:80>
	serverAlias		www.$domain
        ServerName		web.clement.$domain
        ServerAdmin		root@$domain
	RewriteEngine		on
	#comment it in if you want to debug redirect
	#LogLevel                alert rewrite:trace3
        RewriteCond		%{HTTP_HOST} ^www\.clement\.$reldom			[OR]
        RewriteCond		%{HTTP_HOST} ^web\.clement\.$reldom
        RewriteRule		^(.*)$ https://web-clement.$domain	[R=301]
        ErrorLog		logs/wwwmerr.log
        TransferLog		logs/wwwlog.log
</VirtualHost>

<IfDefine needwebmail>
<VirtualHost *:80>
        ServerALias		webmail.$domain
        ServerName		webmail-prod.$domain
        ServerAdmin		root@$domain
	RewriteEngine		on
	#comment it in if you want to debug redirect
	#LogLevel                alert rewrite:trace3
        RewriteCond		%{HTTP_HOST} ^webmail\.$reldom				[OR]
        RewriteCond		%{HTTP_HOST} ^webmail-prod\.$reldom
        RewriteRule		^(.*)$ https://webmail-prod.$domain	[R=301]
        ErrorLog		logs/webmerr.log
        TransferLog		logs/weblog.log
</VirtualHost>
</IfDefine>

<VirtualHost *:443>
        ServerAlias		localhost.localdomain
        ServerAlias		clement.$domain
        ServerName		clement-prod.$domain
        ServerAdmin		root@$domain
        DocumentRoot		$BASE/var/www/$APPNAME-$VERSION
	setenv PATH_INFO	$BASE/etc/$APPNAME-$VERSION/$APPNAME.php
        Alias /base-dir/	$BASE/var/www/$APPNAME-$VERSION/ 
        Alias /reg-icons/	$BASE/var/www/$APPNAME-$VERSION/reg-icons/
        Alias /base-graph/	$BASE/var/www/$APPNAME-$VERSION/graph/
	ScriptAlias /cgi-bin/	$BASE/var/www/$APPNAME-$VERSION/
        <Directory		$BASE/var/www/$APPNAME-$VERSION>
                Options MultiViews FollowSymlinks ExecCGI
                AllowOverride AuthConfig FileInfo Indexes Limit Options
                Order allow,deny
		Allow from all
        </Directory>
	#----------------------------------------------
	#SSL
	SSLEngine On
	SSLCertificateKeyFile	$BASE/etc/pki/$APPNAME-$VERSION/clement.pem
	SSLCertificateFile	$BASE/etc/pki/$APPNAME-$VERSION/clement.pem
	SSLCertificateChainFile $BASE/etc/pki/$APPNAME-$VERSION/StartComClass2PrimaryIntermediateServerCA
	SSLCACertificateFile	$BASE/etc/pki/$APPNAME-$VERSION/startcom.ca.pem
	SSLCipherSuite 		ALL:!ADH:!EXPORT:!SSLv2:RC4+RSA:+HIGH:+MEDIUM:+LOW
	#----------------------------------------------
        ErrorLog logs/clemerr-ssl.log
        TransferLog logs/clemlog-ssl.log
</VirtualHost>

<VirtualHost *:443>
	serverAlias		www.$domain
        ServerName		web.clement.$domain
        ServerAdmin		root@$domain
	DirectoryIndex		menclem.php
	DocumentRoot		$BASE/var/www/$APPNAME-$VERSION/web
	<Directory		$BASE/var/www/$APPNAME-$VERSION/web>
		Options MultiViews FollowSymlinks
		AllowOverride AuthConfig FileInfo Indexes Limit Options
		Order allow,deny
		Allow from all
	</Directory>
	#----------------------------------------------
	#SSL
	SSLEngine On
	SSLCertificateKeyFile	$BASE/etc/pki/$APPNAME-$VERSION/clement.pem
	SSLCertificateFile	$BASE/etc/pki/$APPNAME-$VERSION/clement.pem
	SSLCertificateChainFile $BASE/etc/pki/$APPNAME-$VERSION/StartComClass2PrimaryIntermediateServerCA
	SSLCACertificateFile	$BASE/etc/pki/$APPNAME-$VERSION/startcom.ca.pem
	SSLCipherSuite 		ALL:!ADH:!EXPORT:!SSLv2:RC4+RSA:+HIGH:+MEDIUM:+LOW
	#----------------------------------------------
        ErrorLog		logs/wwwmerr-ssl.log
        TransferLog		logs/wwwlog-ssl.log
</VirtualHost>

<IfDefine needwebmail>
<VirtualHost *:443>
        ServerName		webmail.$domain
        ServerAdmin		root@$domain
	DirectoryIndex		index.php
	DocumentRoot		$BASE/usr/share/squirrelmail
	<Directory		$BASE/usr/share/squirrelmail>
		Options MultiViews FollowSymlinks
		AllowOverride AuthConfig FileInfo Indexes Limit Options
		Order allow,deny
		Allow from all
	</Directory>
	#----------------------------------------------
	#SSL
	SSLEngine On
	SSLCertificateKeyFile	$BASE/etc/pki/$APPNAME-$VERSION/clement.pem
	SSLCertificateFile	$BASE/etc/pki/$APPNAME-$VERSION/clement.pem
	SSLCertificateChainFile $BASE/etc/pki/$APPNAME-$VERSION/StartComClass2PrimaryIntermediateServerCA
	SSLCACertificateFile	$BASE/etc/pki/$APPNAME-$VERSION/startcom.ca.pem
	SSLCipherSuite 		ALL:!ADH:!EXPORT:!SSLv2:RC4+RSA:+HIGH:+MEDIUM:+LOW
	#----------------------------------------------
        ErrorLog		logs/webmerr-ssl.log
        TransferLog		logs/weblog-ssl.log
</VirtualHost>
</IfDefine>

#$APPNAME STOP
!EOT
}

#-----------------------------------------------------------------
#setting name virtual
cat >> $BASE/etc/httpd/conf.d/$APPNAME.conf <<!EOT
#$APPNAME START
#Done by the $APPNAME first start
#NameVirtualHost *:80
#NameVirtualHost *:443
#$APPNAME STOP
!EOT
#to setup the IPV4 WEB interface
if [ -n "$hostipv4" ] ; then
  add_web $hostipv4
  fi

#-----------------------------------------------------------------
#to setup the IPV6 WEB interface
if [ -n "$hostipv6" ] ; then
  add_web $hostipv6
  fi

#-----------------------------------------------------------------
#updating dovecot sql access according data-base type
if [ -n "$LDAP_SERVER" ] ; then
  ln -nsf $BASE/etc/$APPNAME-$VERS/dovecot-1.2-auth-ldap.conf	\
	  $BASE/etc/$APPNAME-$VERS/dovecot-auth.conf
  if [ ! -f /etc/dovecot.conf ] ; then
    ln -nsf $BASE/etc/$APPNAME-$VERS/dovecot-2.0-auth-ldap.conf	\
	    $BASE/etc/$APPNAME-$VERS/dovecot-auth.conf
    fi
  (
  if [ -z "$LDAP_PROTO" -o -z "$LDAP_PORT" ] ; then
    LDAP_PROTO="LDAP";
    LDAP_PORT="389";
    fi
  echo "#Added by addconfig.sh for LDAP authentication" 
  echo "uris= $LDAP_PROTO://$LDAP_SERVER:$LDAP_PORT"
  echo "base= $LDAP_BASE"
  echo ""
  echo "auth_bind = yes"
  echo "pass_filter = (&(objectClass=qmailUser)(accountStatus=active)(|(uid=%u)($LDAP_EMAIL=%u)($LDAP_ALTERNATE_EMAIL=%u)))"
  echo "pass_attrs = uid=user"
  echo ""
  echo "user_filter = (&(objectclass=qmailUser)(accountStatus=active)(|(uid=%u)($LDAP_EMAIL=%u)($LDAP_ALTERNATE_EMAIL=%u)))"
  echo "user_attrs = mailMessageStore=home=$BASE/var/spool/$APPNAME-$VERSION/mails/%$,=mail=maildir:~/dovecot"
  ) >> $BASE/etc/$APPNAME-$VERSION/dovecot-ldap.conf
  (
  echo "UPDATE clmgroup SET "
  echo "authserver='$LDAP_SERVER',authport=$LDAP_PORT,authproto='$LDAP_PROTO',"
  echo "ldapacc='uid=%s,$LDAP_BASE',ldaprootdn='$LDAP_ROODN'"
  case "$DB_TYPE" in 
    "POSTGRESQL"	)
       echo ",ldaprootp=encrypt('$LDAP_PASSWORD','$CODE_CRYPT','aes') "
       ;;
    "MYSQL"	)
       echo ",ldaprootp=encrypt('$LDAP_PASSWORD','$CODE_CRYPT') "
       ;;
    esac
  echo "WHERE groupid=1;"
  ) >> $BASE/etc/$APPNAME-$VERSION/base.sql
else
  (
  echo "#Added by addconfig.sh for $DB_TYPE data-base type" 
  echo "#"`date`
  case "$DB_TYPE" in 
    "POSTGRESQL"	)
      echo "driver = pgsql"
      echo "connect=  dbname=$APPNAME"
      echo "password_query=SELECT password,'Y' AS nopassword FROM users WHERE username='%u'"
      echo "user_query=SELECT 'clement' AS uid,'mail' AS gid,'/var/spool/clement-2.6/mails/%d/%n' AS home,'maildir:~/dovecot' AS mail,'*:storage='||mxmgbytes||'M' AS quota_rule FROM userprofile WHERE prfid=(SELECT prfid FROM users WHERE username='%u')"
      ;;
    "MYSQL"	)
      echo "driver = mysql"
      echo "connect= host=localhost dbname=$APPNAME"
      echo "password_query=SELECT password,'Y' AS nopassword FROM users WHERE username='%u'"
      echo "user_query=SELECT 'clement' AS uid,'mail' AS gid,'/var/spool/clement-2.6/mails/try1.safe.ca/root' AS home,'maildir:~/dovecot' AS mail,concat('*:storage=',mxmgbytes,'M') AS quota_rule from userprofile WHERE prfid=(SELECT prfid FROM users WHERE username='%u')"
      ;;
    esac
  ) >> $BASE/etc/$APPNAME-$VERSION/dovecot-sql.conf
  ln -nsf $BASE/etc/$APPNAME-$VERSION/dovecot-dict-$DB_TYPE.conf.ext $BASE/etc/$APPNAME-$VERSION/dovecot-dict-sql.conf.ext
  ln -nsf $BASE/etc/$APPNAME-$VERS/dovecot-1.2-auth-sql.conf	\
	  $BASE/etc/$APPNAME-$VERS/dovecot-auth.conf
  if [ ! -f /etc/dovecot.conf ] ; then
    ln -nsf $BASE/etc/$APPNAME-$VERS/dovecot-2.0-auth-sql.conf	\
	    $BASE/etc/$APPNAME-$VERS/dovecot-auth.conf
    fi
  case "$DB_TYPE" in 
    "POSTGRESQL"	)
      sed -i -e "s/sqlquota=dbsql/sqlquota=pgsql/" $BASE/etc/$APPNAME-$VERSION/dovecot-2.0.conf
      ;;
    "MYSQL"	)
      sed -i -e "s/sqlquota=dbsql/sqlquota=mysql/" $BASE/etc/$APPNAME-$VERSION/dovecot-2.0.conf
      ;;
    esac
  fi
#-----------------------------------------------------------------
#Setting a randwom string for LDAP ROOT passowrd crypting
sed -i 											\
  -e"s/^CODE_CRYPT=\"your_password\"/CODE_CRYPT=\"`/usr/bin/mkpasswd -s 0 -2`\"/"	\
  $BASE/etc/$APPNAME-$VERSION/$APPNAME.conf
#-----------------------------------------------------------------

