#! /bin/sh
#-------------------------------------------------------------
#procedure to send everyone who had at least a mail in quarantine a warning.
#$1 is the grpout number to scan within emlogs
#-------------------------------------------------------------
#local variable
#originator filed max length
MAXORIG=30
MAXSUBJ=50
#Number of line displayed max per emails
maxline=2000
#Trigger, number of total reference about originator/recipient in the
#quarantine logs after which quarantine is not reported.
MXHITS=4
#-------------------------------------------------------------
#setting the basic definition
. `dirname $0`/../shell/version.sh
#sourcefile: clmsndr -q parameter input file.
sourceDir=$BASE/usr/lib/$APPNAME-$VERS/support/temp
sourceFile=$sourceDir/sendList

#-----------------------------------------------------
#loading the variable value
. $BASE/etc/$APPNAME-$VERSION/$APPNAME.conf
#-----------------------------------------------------

Error='0'
LoggerApp="logger -t clmsndr"


#setting the Variable
#first the Data-base type
if [ -z "$DB_TYPE" ] ; then
  DB_TYPE=POSTGRESQL;
  fi

#Then the data-base name
if [ -z "$DB_NAME" ] ; then
  DB_NAME=$APPNAME;
  fi

#-------------------------------------------------------------
case "$DB_TYPE" in
  "POSTGRESQL"  )
     if [ ! -z "$DB_PORT" ] ; then
       SQLPORT="-p $DB_PORT"
       fi
     if [ ! -z "$DB_HOST" ] ; then
       SQLHOST="-h $DB_HOST"
       fi
     ;;
  "MYSQL"       )
     if [ ! -z "$DB_PORT" ] ; then
       SQLPORT="-P $DB_PORT"
       fi
     if [ ! -z "$DB_HOST" ] ; then
       SQLHOST="-h $DB_HOST"
       fi
     ;;
  esac
#----------------------------------------------
#procedure to probe data-base qith an SQL request
#as $1 parametere
prob_db()

{
case "$DB_TYPE" in
  "POSTGRESQL"  )
     echo "$1" | psql -t -q  -A -F ' ' $SQLPORT $SQLHOST $DB_NAME
     ;;
  "MYSQL"       )
     echo "$1" | mysql -s -u $APPNAME $SQLPORT $SQLHOST $DB_NAME
     ;;
  esac
}

#---------------------------------------------------------
#to extract time in the right format
stampgtz()
#$1 TGZ value

{
case "$DB_TYPE" in
  "POSTGRESQL"  )
    echo "TO_CHAR(stamp AT TIME ZONE '$1','YY-MM-DD HH24:MI:SS')";
    ;;
  "MYSQL"       )
    echo "date_format(convert_tz(stamp,'SYSTEM','$1'),'%Y-%m-%d %H:%i:%s')";
    ;;
  esac
}
#to extract time in the right format
lastscan()
#$1 lastcan value

{
case "$DB_TYPE" in
  "POSTGRESQL"  )
    prob_db "SELECT TO_CHAR('$1'::TIMESTAMP,'YY-MM-DD HH24:MI:SS')";
    ;;
  "MYSQL"       )
    prob_db "SELECT date_format('$1','%Y-%m-%d %H:%i:%s')";
    ;;
  esac
}
#---------------------------------------------------------
#Check the user prfid in order to determine it language.
langCheck()
{
#$1 username

LANG=1;

domain=`echo $1 | cut -d'@' -f2-`;
for TABLE in userprofile domain clmgroup
  do
  case "$TABLE" in
    "userprofile"	)
     case "$DB_TYPE" in
      "POSTGRESQL"  )
	  EMAIL=`echo $1 | sed -e "s/'/''/" `
		  ;;
      "MYSQL"       )
	  EMAIL=`echo $1 | sed -e 's/'\''/\\\\'\''/'`
     ;;
     esac

      lang=`prob_db "SELECT lang FROM $TABLE 		\
			WHERE prfid=			\
			(SELECT prfid FROM users	\
			    WHERE username='$EMAIL');"`
      ;;
    "domain"	)
      grpid=`prob_db "SELECT groupid FROM $TABLE	\
			WHERE domainname='$domain';"`
      lang=`prob_db "SELECT lang FROM $TABLE 		\
			WHERE domainname='$domain';"`
      ;;
    "clmgroup"	)
      lang=`prob_db "SELECT lang FROM $TABLE		\
			WHERE groupid=$grpid;"`
      ;;
    esac
  if [ -n "$lang" ] ; then
    LANG=$lang;
    break;
    fi
  done
case "$LANG" in
  2)			#French
    . $BASE/etc/$APPNAME-$VERS/report_francais.conf
    ;;
  1	|				\
  *)			#English (default)
    . $BASE/etc/$APPNAME-$VERS/report_english.conf
    ;;
  esac
}
#----------------------------------------------
#procedure to insert the originator information
#to be postmaster@`recipient domain`
insertFROM ()
#$1 recipient
{
echo "FROM: $1"
}


#----------------------------------------------
insertRCPT ()
#$1 recipient
{
echo "RCPT: $1"
echo "Mail:"
}



#---------------------------------------------
formatorig ()
#$1 max length
#$2 string

{
data=$2
taille=${#data}
printf " "
if [ $taille -gt $1 ] ; then
  mark=`expr $taille - $1 + 3`
  printf "%*s" $1 "...${data:$mark}"
else
  printf "%*s" $1 "$2"
fi
}
#----------------------------------------------


#----------------------------------------------


formatsubj ()
#$1 max length
#$2 string

{
data=$2
taille=${#data}
if [ $taille -gt $1 ] ; then
  printf "%.$1s" " $data"
else
  printf " %s" $data
fi
}
#----------------------------------------------


formatline ()
#$1 time
#$2 hour
#$3 originator
#$4 subject

{
	echo -n `LANG=$formatLang date --date="$1 $2" +"%b %d %H:%M:%S " `
	formatorig $MAXORIG "$3"
	formatsubj $MAXSUBJ "$4"
	echo 
}


#----------------------------------------------
#checking subject and removing non printable character
#$1 subject
verifySubject ()
{
	if [ "$1" = '(null)' -o -z "$1" ]; then
	   SUBJECT=""
	fi
	SUBJECT=`echo $SUBJECT | tr -d '\000-\037'`
}




#----------------------------------------------




print_mail_header ()
#$1 user
{
        echo "From: "$1
        echo "To: "$1
        echo "$subjectQuar (`LANG=$formatLang TZ=$GTZ date +\"%a, %b %d %H:%M:%S \"`)"
        echo "Date: `date -R`"
        echo "Content-Type: text/plain; charset=UTF-8"
        echo "Content-Transfer-Encoding: 8bit"
}

#----------------------------------------------


print_header ()
#$1 recipient name
{

    case "$DB_TYPE" in
	"POSTGRESQL"  )
	    EMAIL=`echo $1 | sed -e "s/'/''/" `
	    ;;
	"MYSQL"       )
	    EMAIL=`echo $1 | sed -e 's/'\''/\\\\'\''/'`
	    ;;
    esac
realname=`prob_db "SELECT realname FROM users WHERE username='$EMAIL';"`
echo "$salutation $realname,"
echo ""
echo "$header (`LANG=$formatLang TZ=$GTZ  date --date=\"$LASTDATE\" +\"%a,%b %d %H:%M:%S\"`)"
echo ""
echo "$template"
}


#----------------------------------------------


print_quarantines ()
#$1 username
#$2 status

{
	numline=0
     case "$DB_TYPE" in
      "POSTGRESQL"  )
	  EMAIL=`echo $USER | sed -e "s/'/''/" `
		  ;;
      "MYSQL"       )
	  EMAIL=`echo $USER | sed -e 's/'\''/\\\\'\''/'`
     ;;
     esac
	echo ""
	mystamp=`stampgtz $GTZ`;
	prob_db "SELECT $mystamp,			\
			originator,			\
			subject				\
			FROM emlogs			\
			WHERE status=$2 AND		\
			      recipient='$EMAIL' AND	\
		              stamp>'$LASTSCAN'" |
	(while read -r TIME HOUR ORIGINATOR SUBJECT
	  do
	  GOODORIG=`echo $ORIGINATOR | sed -e"s/'/''/"`
	  hits=`prob_db "SELECT COUNT(*) FROM emlogs	\
			 WHERE originator='$GOODORIG'	\
			      AND recipient='$EMAIL'	\
			      AND status=$2"`
          if [ $hits -le $MXHITS ] ; then
	    numline=`expr $numline + '1'`
	    if [ $numline -le $maxline ] ; then 
	      verifySubject \"$SUBJECT\"
	      formatline $TIME $HOUR $ORIGINATOR "$SUBJECT"
	      fi	
	    if [ $numline -eq $maxline ] ; then  
	      echo "[...]"
	      fi
	    fi
	  done

	  if [ $numline -ge $maxline ] ; then
	  echo "$total $numline $warnThreshold"
	  fi
)
}

#----------------------------------------------



print_footer ()

{
echo ""
echo "$footer"
}

#----------------------------------------------


print_email ()

{
#$1 user
#$2 status


DBUSR="users"
USERS="name"
case "$2" in
  "16"	)		#Internal user 
    DBUSR="intusers"
    USERS="intname"
    ;;
			#Local user & unexpected status
  "2"   |					\
  "*"   )
    ;;
  esac

RANDOM=$$$(date +%s)
DYNAKEY=`printf "%010d" $RANDOM$RANDOM; expr \`date +%N\` / 1000`
NAME=`echo $1 | cut -d'@' -f1`
DOMAIN=`echo $1 | cut -d'@' -f2`
GOCLEM="http://$CLEMSRVR/access.php?&$USERS=$NAME&domain=$DOMAIN&dynakey=$DYNAKEY"
     case "$DB_TYPE" in
      "POSTGRESQL"  )
	  EMAIL=`echo $1 | sed -e "s/'/''/" `
		  ;;
      "MYSQL"       )
	  EMAIL=`echo $1 | sed -e 's/'\''/\\\\'\''/'`
     ;;
     esac

langCheck $1

insertFROM $1
insertRCPT $1
print_mail_header $1 
#header separator
echo ""			
print_header $1
print_quarantines $1 $2
print_footer
#Email End
echo "."
prob_db "UPDATE $DBUSR SET dynamickey='$DYNAKEY' WHERE username='$EMAIL';"
}
#----------------------------------------------
#scanning from group list
for GRP in $*
  do
  export GTZ=`prob_db "SELECT timezone FROM clmgroup WHERE groupid=$GRP;"`
  export CLEMSRVR=`prob_db "SELECT clemserver FROM clmgroup WHERE groupid=$GRP;"`
  export LASTSCAN=`prob_db "SELECT quarantine FROM clmgroup WHERE groupid=$GRP;"`
  export LASTDATE=`lastscan "$LASTSCAN"`
  #checking for quarantine for
  #	Local user (status=2)
  #	Internal user (status=16)
  for STATUS in 2 16
    do
    USERS=`prob_db "SELECT DISTINCT recipient FROM emlogs WHERE 		\
			status=$STATUS AND stamp>'$LASTSCAN' AND 		\
			grpout=$GRP ORDER BY recipient;"`

    for USER in $USERS	   
      do
     case "$DB_TYPE" in
      "POSTGRESQL"  )
	  EMAIL=`echo $USER | sed -e "s/'/''/" `
		  ;;
      "MYSQL"       )
	  EMAIL=`echo $USER | sed -e 's/'\''/\\\\'\''/'`
     ;;
     esac

     if [ $STATUS == 2 ] ; then	# check active local user ONLY
        #do not send quarantine report to not active user
        active=`prob_db "SELECT COUNT(*) FROM users WHERE 			\
			  username='$EMAIL' AND active=1"`
        if [ $active -eq 0 ] ; then
	  continue;
          fi
 	fi
      hits=`prob_db "SELECT COUNT(*) FROM emlogs WHERE				\
			recipient='$EMAIL' AND status=$STATUS AND		\
		        stamp>'$LASTSCAN'"`
      if [ $hits -gt 0 ] ; then
        print_email $USER $STATUS
        fi
      done
    done
  prob_db "UPDATE clmgroup SET quarantine=NOW() WHERE groupid=$GRP;"
  done
