#! /bin/sh
#----------------------------------------------------------------------
#program to check everything is ready to start clement
#$1 is either "sysvinit" or "systemd"
#----------------------------------------------------------------------
#Hard coded variables
APPNAME=clement
VERS=2.6
PLOCK=/var/lock/subsys
#----------------------------------------------------------------------
if [ "$1" = "sysvinit" ] ; then
  [ -f  /etc/rc.d/init.d/functions ] && . /etc/rc.d/init.d/functions
  fi;
#----------------------------------------------------------------------
#all applications variables
[ -f /etc/$APPNAME-$VERS/$APPNAME.conf ] && . /etc/$APPNAME-$VERS/$APPNAME.conf
#overiding some variable by local setting
[ -f /etc/sysconfig/$APPNAME ] && . /etc/sysconfig/$APPNAME
#----------------------------------------------------------------------
#exiting with a status
#$1 exit status
sh_badexit()

{
if [ "$1" != "systemd" ] ; then
  echo -n "Bad exit status $2" ; failure;
  echo
else
  echo
  echo "Exiting with fatal status=$2"
fi
exit $1
}
#----------------------------------------------------------------------
#restarting a daemon
#$1 init mode "sysvinit" or "systemd"
#$2 daemon application name
sh_restart()

{
if [ "$1" != "systemd" ] ; then
  echo
  service $2 restart
else
  systemctl restart $2
fi
}
#----------------------------------------------------------------------
#reporting a big trouble
sh_failure()

{
if [ "$1" != "systemd" ] ; then
  echo -n $2 ; failure;
  echo
else
  echo
  echo Failure: $1
fi
}
#----------------------------------------------------------------------
#reporting a small problem
sh_warning()

{
if [ "$1" != "systemd" ] ; then
  echo -n $2 ; warning;
  echo
else
  echo
  echo Warning: $2
fi
}
#----------------------------------------------------------------------
#reporting a success
sh_success()

{
if [ "$1" != "systemd" ] ; then
  echo -n $2 ; success;
  echo
else
  echo
  echo Success: $2
fi
}
#----------------------------------------------------------------------
#adding dovecot configuration
#----------------------------------------------------------------------
do_doveconf()

{
export	BEFCLEM=/etc/dovecot.conf.before.$APPNAME
export	DOVCONF=/etc/dovecot.conf
export  NEWCONF=/etc/$APPNAME-$VERS/dovecot-1.2.conf
ln -nsf	/etc/$APPNAME-$VERS/dovecot-1.2-auth-sql.conf	\
	/etc/$APPNAME-$VERS/dovecot-auth-sql.conf
ln -nsf	/etc/$APPNAME-$VERS/dovecot-1.2-auth-ldap.conf	\
	/etc/$APPNAME-$VERS/dovecot-auth-ldap.conf

if [ ! -f $DOVCONF ] ; then
  export BEFCLEM=/etc/dovecot/dovecot.conf.before.$APPNAME
  export DOVCONF=/etc/dovecot/dovecot.conf
  export NEWCONF=/etc/$APPNAME-$VERS/dovecot-2.0.conf
  ln -nsf	/etc/$APPNAME-$VERS/dovecot-2.0-auth-sql.conf	\
		/etc/$APPNAME-$VERS/dovecot-auth-sql.conf
  ln -nsf	/etc/$APPNAME-$VERS/dovecot-2.0-auth-ldap.conf	\
		/etc/$APPNAME-$VERS/dovecot-auth-ldap.conf
  fi

if [ -f $DOVCONF ] ; then
  if [ ! -f $BEFCLEM ] ; then
    mv $DOVCONF $BEFCLEM
    ln -s $NEWCONF $DOVCONF
    ln -s $DOVCONF /etc/$APPNAME-$VERS/dovecot.conf
    cat  /etc/$APPNAME-$VERS/dovecot-auth.conf >> $DOVCONF
    fi
  fi
}
#----------------------------------------------------------------------
#building application data-base
#----------------------------------------------------------------------
do_mkdb()

{
case "$DB_TYPE" in
  "MYSQL"	)
    #grabbing temporary control on the MySQL daemon
    service mysqld stop
    /usr/bin/mysqld_safe -u root --skip-grant-tables >> /etc/$APPNAME-$VERS/dbinst.log 2>&1 &
    sleep 5		#give time to daemon to start
    #now we have control over mysql daemon
    (
    echo "CREATE TABLE user_bck AS (SELECT * FROM user);"
    echo "UPDATE user SET Password=NULL WHERE User='root' AND Host='localhost';"
    ) | mysql -u root mysql >> /etc/$APPNAME-$VERS/dbinst.log
    ( 
    echo "Shutdown mysqld skip-grant-tables mode" 
    /usr/bin/mysqladmin -u root shutdown
    echo "starting mysqld daemon" 
    service mysqld start
    )>> /etc/$APPNAME-$VERS/dbinst.log 2>&1
    /usr/lib/$APPNAME-$VERS/support/crdb.sh "/"
    (
    echo "UPDATE user"
    echo "SET Password=(SELECT Password FROM user_bck WHERE User='root' AND Host='localhost')"
    echo "WHERE User='root' AND Host='localhost';"
    echo "DROP TABLE user_bck;"
    ) | mysql -u root mysql
    ;;
  "POSTGRESQL"	)
    /usr/lib/$APPNAME-$VERS/support/crdb.sh "/"
    ;;
  "NOT_DEFINED"	)
    echo "You must install $APPNAME with a DB flavor (ex: $APPNAME-pgsql, $APPNAME-mysql)"
    echo "Exiting now"
    exit -2;
    ;;
  "*"		)
    echo "Unexpectd DB_TYPE variable within /etc/sysconfig/$APPNAME, please check"
    echo "Exiting now"
    exit -3;
    ;;
  esac
/usr/lib/$APPNAME-$VERS/support/mkdb.sh "/"
chown $APPNAME /etc/$APPNAME-$VERS/dbinst.sql
chown $APPNAME /etc/$APPNAME-$VERS/dbinst.log
(
export hostipv4=`dig \`uname -n\` A +noall +short`;
export hostipv6=`dig \`uname -n\` AAAA +noall +short`;
#to make sure we have at least an hostipv4 reference
if [ -z "$hostipv4" -a -z "$hostipv6" ] ; then
  export hostipv4=`grep \`uname -n\` /etc/hosts | cut  -f1`
  fi

/bin/su - -m $APPNAME << !EOT
export SMTPPORT0=25;
export SMTPPORT1=587;
export SMTPPORT3=465;
export SMTPIPV4=$hostipv4;
export SMTPIPV6=$hostipv6;
export RPASS=`grep root /etc/shadow | cut -d':' -f2 | sed -es/\\\\$/\\\\\\\\$/g`;
export DOMAINNAME=`dnsdomainname`;
export HOSTNAME=`hostname`;
export SPAMID=$APPNAME;
/usr/lib/$APPNAME-$VERS/support/clmmkdb.sh "" /etc/$APPNAME-$VERS/base.sql
!EOT
)
}
#----------------------------------------------------------------------
#procedure to check if the config is properly done
#you MUST AT LEAST, specify the Data Base type to be used by clement.
#----------------------------------------------------------------------
chk_config()

{
if      [ -z "$DB_TYPE" -o "$DB_TYPE" = "TO_BE_DEFINED" ] ; then
  echo -n "Checking file /etc/$APPNAME-$VERS/$APPNAME.conf"
  sh_failure $1 $"/etc/$APPNAME-$VERS/$APPNAME.conf must be configured"
  echo -n "Please set DB_TYPE within /etc/$APPNAME-$VERS/$APPNAME.conf"
  sh_warning $1 $"DB_TYPE in /etc/$APPNAME-$VERS/$APPNAME.conf must be set"
  exit -1;
  fi
}
#----------------------------------------------------------------------
#procedure to check if we are able to find at least one IPV4 number
#linked to system name
#----------------------------------------------------------------------
chk_uname()

{
hostipv4=`dig \`uname -n\` A +noall +short`;
hostipv6=`dig \`uname -n\` AAAA +noall +short`;
#to make sure we have at least an hostipv4 reference
if [ -z "$hostipv4" -a -z "$hostipv6" ] ; then
  hostipv4=`grep \`uname -n\` /etc/hosts | cut  -f1`
  fi
#double checking if we have an IPV4
if [ -z "$hostipv4" -a -z "$hostipv6" ] ; then
  echo "Unable to find a IP number link to this server "`uname -n`
  echo "Fix your DNS or your /etc/hosts file, to have at least one IP number"
  echo "returned by next line command line"
  echo "> grep \`uname -n\` /etc/hosts | cut  -f1"
  echo "Exiting now"
  exit -1;
  fi
}
#----------------------------------------------------------------------
#procedure to check if data-base server is available
#----------------------------------------------------------------------
chk_dbsrv()

{
case "$DB_TYPE" in
  "MYSQL"       )
    up="false"
    if [ "$1" == "systemd" ] ; then
      active=`systemctl is-active mysqld`
      if [ "$active" == "active" ] ; then
	up="true"
	fi
      else
      if [ -f $PLOCK/mysqld ] ; then
        up="true"
	fi
      fi
    if [ "$up" == "false" ] ; then
      echo "You must have mysql daemon (mysqld) up and running before to start clement"
      if [ "$1" == "systemd" ] ; then
        echo "See: \"systemctl start mysqld\""
        else
        echo "See: \"service mysqld start\""
        fi
      echo "Exiting now"
      exit -1;
      fi
    ;;
  "POSTGRESQL"  )
    up="false"
    if [ "$1" == "systemd" ] ; then
      active=`systemctl is-active postgresql`
      if [ "$active" == "active" ] ; then
	up="true"
	fi
      else
      if [ -f /var/lib/pgsql/data/postmaster.pid ] ; then
        up="true"
	fi
      fi
    if [ "$up" == "false" ] ; then
      echo "You must have postgresql daemon (postmaster) up and running before to start clement"
      if [ "$1" == "systemd" ] ; then
        echo "See: \"systemctl start postgresql\""
        else
        echo "see: \"service postgresql start\""
        fi
      echo "Exiting now"
      exit -1;
      fi
    ;;
  "NOT_DEFINED" )
    echo "You must have a data-base daemon with a DB flavor (ex: postgresql, mysqld) defined"
    echo "See /etc/sysconfig/$APPNAME file"
    echo "Exiting now"
    exit -2;
    ;;
  "*"           )
    echo "Unexpectd DB_TYPE variable within /etc/sysconfig/$APPNAME, please check"
    echo "Exiting now"
    exit -3;
    ;;
  esac
}
#----------------------------------------------------------------------
#creating clement local certificat
#----------------------------------------------------------------------
do_mkcert()

{
        if [ ! -f /etc/pki/$APPNAME-$VERS/$APPNAME.pem ] ; then
          echo -n $"Preparing $APPNAME certificat: "
          /usr/lib/$APPNAME-$VERS/support/dummy-cert \
                 /etc/pki/$APPNAME-$VERS/$APPNAME.pem
          chown $APPNAME.mail /etc/pki/$APPNAME-$VERS/$APPNAME.pem
          sh_success $"certs generation"
          echo
          fi
}
#----------------------------------------------------------------------
#setting local clement configuration
#----------------------------------------------------------------------
do_mkconf()

{
if [ ! -f /etc/$APPNAME-$VERS/config.done ] ; then
  /usr/lib/$APPNAME-$VERS/support/addconfig.sh
  sh_success $1 $"conf addition"
  do_mkdb ;
  do_doveconf ;
  sh_restart $1 httpd
  if [ "$1" == "systemd" ] ; then
    (
    echo "#added by clement install"
    echo "[Unit]"
    case "$DB_TYPE" in
      "MYSQL"		)
	echo "After=mysqld.service"
	echo "Requires=mysqld.service"
        ;;
      "POSTGRESQL"	)
	echo "After=postgresql.service"
	echo "Requires=postgresql.service"
        ;;
      "*"		)
	echo "#After=No database"
	echo "#Requires=No database"
        ;;
      esac
    ) >> /etc/systemd/system/$APPNAME.service
    systemctl --system daemon-reload
    fi
  date > /etc/$APPNAME-$VERS/config.done
fi
}
#----------------------------------------------------------------------
#procedure to check if spamassassin is up and running
#----------------------------------------------------------------------
chk_spamd()

{
if [ "$1" != "systemd" ] ; then
  if [ ! -f $PLOCK/spamd -a ! -f $PLOCK/spamassassin ] ; then
    sh_warning $1 $"Spamassassin daemon is NOT found up and running!"
    echo $"use 'chkconfig spamassassin on' to start spammassin each time"
    fi
  fi
}
#----------------------------------------------------------------------
#procedure to check if freshclam and clamd are up and running
#----------------------------------------------------------------------
chk_clam()

{
if [ "$1" != "systemd" ] ; then
  if [ ! -f $PLOCK/clamd ] ; then
    sh_warning $1 $"clamd daemon is NOT found up and running!"
    echo "use 'chkconfig clamd on' to start spammassin each time"
    fi
  fi
}
#----------------------------------------------------------------------
#main program

chk_config $1;
chk_uname;
chk_dbsrv $1;
do_mkcert;
do_mkconf $1;
chk_spamd $1;
chk_clam $1;
/usr/lib/$APPNAME-$VERS/support/genphp.sh
exit 0;
